/*
 * Filesystem provider registry.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "provider_registry.h"

/* To complete forward declaration. */

#include "provider.h"



/* Methods for use with the lock already acquired. */

/* Return any registered provider for the given 'fileid' or NULL if no such
   provider is registered. */

Provider *ProviderRegistry::get(fileid_t fileid)
{
    ProviderMapping::iterator entry = _providers.find(fileid);
    Provider *provider;

    if (entry == _providers.end())
        provider = NULL;
    else
        provider = entry->second;

    return provider;
}

/* Remove a provider and its resources for the given 'fileid'. */

void ProviderRegistry::remove(fileid_t fileid, Provider *provider)
{
    _providers.erase(fileid);
    delete provider;
}

/* Register a 'provider' for the given 'fileid'. */

void ProviderRegistry::set(fileid_t fileid, Provider *provider)
{
    ProviderMapping::iterator entry = _providers.find(fileid);

    if (entry != _providers.end())
        return;

    _providers[fileid] = provider;
}



/* Detach from a provider, potentially removing it from the registry. */

void ProviderRegistry::detach(fileid_t fileid, Provider *provider)
{
    std::lock_guard<std::mutex> guard(_lock);

    if (!provider->detach())
        remove(fileid, provider);
}

// vim: tabstop=4 expandtab shiftwidth=4
